@@ -194,21 +194,8 @@ class WP_Object_Cache {
 				foreach ($dogs as $catt)
 					$this->cache['category'][$catt->cat_ID] = $catt;
 			}
-		} else
-			if ('options' == $group) {
-				$wpdb->hide_errors();
-				if (!$options = $wpdb->get_results("SELECT option_name, option_value FROM $wpdb->options WHERE autoload = 'yes'")) {
-					$options = $wpdb->get_results("SELECT option_name, option_value FROM $wpdb->options");
-				}
-				$wpdb->show_errors();
+		}
 
-				if ( ! $options )
-					return;
-
-				foreach ($options as $option) {
-					$this->cache['options'][$option->option_name] = $option->option_value;
-				}
-			}
 	}
 
 	function make_group_dir($group, $perms) {
@@ -353,10 +340,9 @@ class WP_Object_Cache {
 				fputs($fd, $serial);
 				fclose($fd);
 				if (!@ rename($temp_file, $cache_file)) {
-					if (@ copy($temp_file, $cache_file))
-						@ unlink($temp_file);
-					else
+					if (!@ copy($temp_file, $cache_file))
 						$errors++;
+					@ unlink($temp_file);
 				}
 				@ chmod($cache_file, $file_perms);
 			}
@@ -399,7 +385,7 @@ class WP_Object_Cache {
 	function WP_Object_Cache() {
 		return $this->__construct();
 	}
-	
+
 	function __construct() {
 		global $blog_id;
 
@@ -442,7 +428,7 @@ class WP_Object_Cache {
 
 	function __destruct() {
 		$this->save();
-		return true;	
+		return true;
 	}
 }
 ?>
